package com.cirs.biz.service;

import java.util.List;

import com.cirs.biz.domain.TVerificationTaskDetail;
import com.cirs.biz.domain.VerificationData;

/**
 * 验证任务详情Service接口
 * 
 * @author allen
 * @date 2023-11-03
 */
public interface ITVerificationTaskDetailService 
{
    /**
     * 查询验证任务详情
     * 
     * @param id 验证任务详情主键
     * @return 验证任务详情
     */
    public TVerificationTaskDetail selectTVerificationTaskDetailById(Long id);

    /**
     * 查询验证任务详情列表
     * 
     * @param tVerificationTaskDetail 验证任务详情
     * @return 验证任务详情集合
     */
    public List<TVerificationTaskDetail> selectTVerificationTaskDetailList(TVerificationTaskDetail tVerificationTaskDetail);

    /**
     * 新增验证任务详情
     * 
     * @param tVerificationTaskDetail 验证任务详情
     * @return 结果
     */
    public int insertTVerificationTaskDetail(TVerificationTaskDetail tVerificationTaskDetail);

    /**
     * 修改验证任务详情
     * 
     * @param tVerificationTaskDetail 验证任务详情
     * @return 结果
     */
    public int updateTVerificationTaskDetail(TVerificationTaskDetail tVerificationTaskDetail);

    /**
     * 批量删除验证任务详情
     * 
     * @param ids 需要删除的验证任务详情主键集合
     * @return 结果
     */
    public int deleteTVerificationTaskDetailByIds(Long[] ids);

    /**
     * 删除验证任务详情信息
     * 
     * @param id 验证任务详情主键
     * @return 结果
     */
    public int deleteTVerificationTaskDetailById(Long id);

//    public String importTaskDetail(List<TVerificationTaskDetail> taskDetailList, boolean updateSupport, String operName);

    String importTaskDetail(List<TVerificationTaskDetail> taskDetailList, boolean updateSupport, String operName, String fileName);

    public List<TVerificationTaskDetail> selectTVerificationTaskDetailByTaskIds(List<Long> taskids);

    public List<TVerificationTaskDetail> alldata(Long taskid);

    public Long setComponentId(String result);

    List<VerificationData> getComponentids(Long taskid);
}
